@@ -6,6 +6,7 @@ from .utils import _get_default_mmc, _default_rf_classifier, _default_rf_regress
 from pandas import DataFrame
 import numpy as np
 from typing import Union, List, Dict, Any, TYPE_CHECKING
+from .logger import Logger
 
 if TYPE_CHECKING:
     from sklearn.ensemble import RandomForestClassifier, RandomForestRegressor
@@ -94,16 +95,13 @@ class KernelDataSet(ImputedDataSet):
 
         # Format mean_match_candidates before priming datasets
         available_candidates = {
-            var: self.data_shape[0] - self.data[var].isna().sum()
-            for var in self.response_vars
+            var: (-self.data[var].isna()).sum() for var in self.response_vars
         }
-
         if self.mean_match_candidates is None:
             self.mean_match_candidates = {
                 var: _get_default_mmc(available_candidates[var])
                 for var in self.response_vars
             }
-
         elif isinstance(self.mean_match_candidates, int):
             self.mean_match_candidates = {
                 key: self.mean_match_candidates for key in self.response_vars
@@ -115,6 +113,8 @@ class KernelDataSet(ImputedDataSet):
                     + "Do all variables in variable_schema have missing values?."
                 )
 
+        # Make sure there are enough non-missing values
+        # in each column to pull candidates from.
         mmc_inadequate = [
             var
             for var, mmc in self.mean_match_candidates.items()
@@ -128,9 +128,8 @@ class KernelDataSet(ImputedDataSet):
                 + " do not have enough candidates to perform mean matching."
             )
 
-        # Initialize models
+        # Initialize models and time_log
         self.models: Dict[str, Dict] = {var: {0: None} for var in self.response_vars}
-
         self.time_log = TimeLog(_TIMED_EVENTS)
 
     def __repr__(self):
@@ -237,7 +236,7 @@ class KernelDataSet(ImputedDataSet):
             candidate_values = np.array(candidate_values)
 
             # Determine the nearest neighbors of the bachelor predictions in the candidate predictions
-            knn = NearestNeighbors(n_neighbors=mmc, algorithm="ball_tree")
+            knn = NearestNeighbors(n_neighbors=mmc, algorithm="ball_tree", n_jobs=-1)
             knn.fit(candidate_preds.reshape(-1, 1))
             knn_indices = knn.kneighbors(
                 bachelor_preds.reshape(-1, 1), return_distance=False
@@ -282,6 +281,8 @@ class KernelDataSet(ImputedDataSet):
 
         """
 
+        logger = Logger(verbose)
+
         mice_s = datetime.now()
 
         iterations_at_start = self.iteration_count()
@@ -293,11 +294,9 @@ class KernelDataSet(ImputedDataSet):
         assert isinstance(self.mean_match_candidates, Dict)
 
         for iteration in iter_range:
-            if verbose:
-                print(str(iteration) + " ", end="")
+            logger.log(str(iteration) + " ", end="")
             for var in self.response_vars:
-                if verbose:
-                    print(" | " + var, end="")
+                logger.log(" | " + var, end="")
 
                 x, y = self._make_xy(var=var)
                 non_missing_ind = self.na_where[var] == False
@@ -345,8 +344,7 @@ class KernelDataSet(ImputedDataSet):
                     )
 
                 self._insert_new_data(var, imp_values)
-            if verbose:
-                print("\n", end="")
+            logger.log("\n", end="")
         self.time_log.add_time("mice", mice_s)
 
     def impute_new_data(
@@ -391,6 +389,8 @@ class KernelDataSet(ImputedDataSet):
 
         """
 
+        logger = Logger(verbose)
+
         impute_new_data_s = datetime.now()
 
         if set(new_data.columns) != set(self.data.columns):
@@ -416,8 +416,7 @@ class KernelDataSet(ImputedDataSet):
         assert isinstance(self.mean_match_candidates, Dict)
 
         for iteration in iter_range:
-            if verbose:
-                print(str(iteration) + " ", end="")
+            logger.log(str(iteration) + " ", end="")
 
             # Determine which model iteration to grab
             if self.save_models == 1 or iteration > curr_iters:
@@ -426,7 +425,7 @@ class KernelDataSet(ImputedDataSet):
                 itergrab = iteration
 
             for var in iter_vars:
-                print(" | " + var, end="")
+                logger.log(" | " + var, end="")
 
                 x, y = imputed_data_set._make_xy(var)
                 kernelx, kernely = self._make_xy(var)
@@ -456,8 +455,7 @@ class KernelDataSet(ImputedDataSet):
                     )
 
                 imputed_data_set._insert_new_data(var, imp_values)
-            if verbose:
-                print("\n", end="")
+            logger.log("\n", end="")
         self.time_log.add_time("impute_new_data", impute_new_data_s)
         return imputed_data_set
 
